from copy import copy
import os
import shutil
import sys
import urllib.parse as urlparse

SUPPORTED_PIP = "19."
SUPPORTED_POETRY = "0.12."


def run_patch(self, *args, **kwargs):
    """
    Patch function for poetry's PipInstaller.run
    """

    # create args buffer
    fixed_args = list(copy(args))

    # search for --index-url arg
    for i in range(len(args)):
        if args[i] == "--index-url" or args[i] == '-i':
            # fix creds in url
            try:
                fixed_args[i+1] = fix_creds(args[i+1])
            except Exception as e:
                sys.exit(f'Failed to fix credentials: {str(e)}')

    # call unpatched PipInstaller.run
    self._unpatched_run(*fixed_args, **kwargs)


# fix_creds
def fix_creds(url):
    """
    Encode credentials in a `url` containing unencoded credentials
    """
    # process url with urlparse
    parsed_url = urlparse.urlparse(url)

    # separate netloc into creds and host
    try:
        creds, host = parsed_url.netloc.rsplit('@', 1)
    except ValueError:
        # don't modify the url if there are no creds
        return url

    # separate creds into user and password
    user, password = creds.split(':', 1)

    # urlencode user and password
    fixed_creds = f'{urlparse.quote(user)}:{urlparse.quote(password)}'

    # reconstruct netloc with urlencoded creds
    fixed_url = parsed_url._replace(netloc=f'{fixed_creds}@{host}')

    # return fixed url
    return fixed_url.geturl()


def main():
    # add poetry dir to libdir so we can import it
    path = shutil.which("poetry")

    # exit with error if path is null
    if not path:
        sys.exit('Installation for poetry was not found')

    dirname = os.path.dirname(path)
    libdir = f'{dirname}/../lib'
    sys.path.insert(0, libdir)

    from poetry import console as poetry_console
    from poetry.installation.pip_installer import PipInstaller

    # check stuff that would disable patching
    try:
        # this will raise an exception if pip isn't installed, let poetry handle that error
        import pip

        # if pip version is not supported, raise exception
        if not pip.__version__.startswith(SUPPORTED_PIP):
            raise Exception()

        # if poetry version is not supported, raise exception
        if not poetry.__version__.startswith(SUPPORTED_POETRY):
            raise Exception()

        # save PipInstaller.run
        setattr(PipInstaller, '_unpatched_run', PipInstaller.run)

        # patch PipInstaller.run
        PipInstaller.run = run_patch

    except:
        # do nothing
        pass

    # call poetry
    poetry_console.main()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['pooetry']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['pooetry = pooetry:main.main']}

setup_kwargs = {
    'name': 'pooetry',
    'version': '0.1.2',
    'description': 'An almost-useless wrapper for the great packaging tool poetry',
    'long_description': "# What is this?\n`pooetry` is a python wrapper utility with a very narrow use case.  If you don't know what it does you probably don't need it 🙃\n\n# What does it do?\nThis utility automagically fixes issues caused by these factors combined:\n* `poetry 0.12.*`\n* `pip 19.*`\n* Use a private package repo which requires authentication\n* Have special characters in your credentials for this private repo\n    * _commonly caused by having an email as a username (the '@' is not URL legal)_\n\nIf you are stuck in this situation you'll find yourself changing the credentials between quoted and non-quoted as _some_ `poetry` commands only work with quoted creds while _others_ only work with non-quoted creds.\n\n# How do I use it?\n1. Install `poetry`\n    * Best to follow the [install procedure on the official `poetry` repo](https://github.com/sdispater/poetry#installation)\n2. Install `pooetry`\n    * You can use the package manager of your choice for this, `pip install pooetry` for example\n3. When you want to run `poetry` simply use `pooetry` instead.  All the commands and options will be passed through.",
    'author': 'ChrisArgyle',
    'author_email': 'chrisisdiy@gmail.com',
    'url': 'https://github.com/ChrisArgyle/pooetry',
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
